package org.greenstone.gatherer.metadata;


import org.greenstone.gatherer.util.XMLTools;
import org.w3c.dom.*;


/** This class represents one Metadata element */
public class MetadataElement
{
    private String metadata_element_name_full = null;
    private Element metadata_element_element = null;
    private MetadataValueTreeModel metadata_value_tree_model = null;


    public MetadataElement(String metadata_element_name_full, Element metadata_element_element)
    {
	this.metadata_element_name_full = metadata_element_name_full;
	this.metadata_element_element = metadata_element_element;
	this.metadata_value_tree_model = new MetadataValueTreeModel(this);
    }


    public MetadataValueTreeNode addMetadataValue(String metadata_value)
    {
	return metadata_value_tree_model.addMetadataValue(metadata_value);
    }


    public String getAttribute(String attribute_name, String language_code)
    {
	// Some extracted metadata elements may not be in ex.mds, and so have no attributes
	if (metadata_element_element == null) {
	    return null;
	}

	Element parent_element = metadata_element_element;
	boolean language_matches = false;

	NodeList language_elements_nodelist = metadata_element_element.getElementsByTagName("Language");
	for (int i = 0; i < language_elements_nodelist.getLength(); i++) {
	    Element current_language_element = (Element) language_elements_nodelist.item(i);
	    String current_language_element_code = current_language_element.getAttribute("code");
	    if (current_language_element_code.equals(language_code)) {
		// Found language; use it for the next step
		parent_element = current_language_element;
		language_matches = true;
		break;
	    }
	}

	NodeList attribute_elements_nodelist = parent_element.getElementsByTagName("Attribute");
	for (int i = 0; i < attribute_elements_nodelist.getLength(); i++) {
	    Element current_attribute_element = (Element) attribute_elements_nodelist.item(i);
	    String current_attribute_element_name = current_attribute_element.getAttribute("name");
	    if (!current_attribute_element_name.equals(attribute_name)) {
		// Move on to the next Attribute element if the name doesn't match
		continue;
	    }

	    // Check if the language matches
	    String current_attribute_element_language = current_attribute_element.getAttribute("language");
	    if (language_matches || current_attribute_element_language.equals(language_code)) {
		// Found attribute; returns its value
		return XMLTools.getElementTextValue(current_attribute_element);
	    }
	}

	return null;
    }


    public MetadataValueTreeModel getMetadataValueTreeModel()
    {
	return metadata_value_tree_model;
    }


    public MetadataValueTreeNode getMetadataValueTreeNode(String metadata_value)
    {
	return metadata_value_tree_model.getMetadataValueTreeNode(metadata_value);
    }


    public String getFullName()
    {
	return metadata_element_name_full;
    }


    public String getNamespace()
    {
	return MetadataTools.getMetadataSetNamespace(metadata_element_name_full);
    }


    public String getName()
    {
	return MetadataTools.getMetadataElementName(metadata_element_name_full);
    }


    public boolean isExtractedMetadata()
    {
	return getNamespace().equals(MetadataSetManager.EXTRACTED_METADATA_NAMESPACE);
    }


    /** This is used to display the element in the metadata value table */
    public String toString()
    {
	return metadata_element_name_full;
    }
}
